@@ -10,12 +10,12 @@ from ncpol2sdpa import generate_variables, SdpRelaxation, solve_sdp,\
     projective_measurement_constraints
 
 
-def expectation_values(M, outcomes):
+def expectation_values(measurement, outcomes):
     exp_values = []
-    for k in range(len(M)):
+    for k in range(len(measurement)):
         exp_value = 0
-        for i in range(len(M[k])):
-            exp_value += outcomes[k][i] * M[k][i]
+        for j in range(len(measurement[k])):
+            exp_value += outcomes[k][j] * measurement[k][j]
         exp_values.append(exp_value)
     return exp_values
 
@@ -29,7 +29,7 @@ E = generate_variables(n_vars, name='E', hermitian=True)
 
 # Define measurements and outcomes
 M, outcomes = [], []
-for i in range(n_vars / 2):
+for i in range(int(n_vars / 2)):
     M.append([E[2 * i], E[2 * i + 1]])
     outcomes.append([1, -1])
 
@@ -45,4 +45,4 @@ objective = -(C[0] * C[2] + C[0] * C[3] + C[1] * C[2] - C[1] * C[3])
 
 sdpRelaxation = SdpRelaxation(E, verbose=2)
 sdpRelaxation.get_relaxation(objective, [], [], monomial_substitutions, level)
-print solve_sdp(sdpRelaxation)
+print(solve_sdp(sdpRelaxation))
