@@ -12,13 +12,12 @@ Created on Fri May 10 09:45:11 2013
 @author: Peter Wittek
 """
 
-from ncpol2sdpa.ncutils import generate_variables
-from ncpol2sdpa.sdprelaxation import SdpRelaxation
+from ncpol2sdpa import generate_variables, SdpRelaxation, write_to_sdpa
 
 # Number of Hermitian variables
 n_vars = 2
-# Order of relaxation
-order = 2
+# Level of relaxation
+level = 2
 
 # Get Hermitian variables
 X = generate_variables(n_vars, hermitian=True)
@@ -39,5 +38,5 @@ monomial_substitution[X[0] ** 2] = X[0]
 # Obtain SDP relaxation
 sdpRelaxation = SdpRelaxation(X)
 sdpRelaxation.get_relaxation(obj, inequalities, equalities,
-                             monomial_substitution, order)
-sdpRelaxation.write_to_sdpa('example_noncommutative.dat-s')
+                             monomial_substitution, level)
+write_to_sdpa(sdpRelaxation, 'example_noncommutative.dat-s')
