@@ -2,7 +2,7 @@
 """
 Exporting a Hamiltonian ground state problem to SDPA. The Hamiltonian
 is of a simple harmonic oscillator. Bosonic systems reach the optimum
-solution at relaxation order 1:
+solution at relaxation level 1:
 
 Navascués, M. García-Sáez, A. Acín, A. and Pironio, S. A paradox in bosonic
 energy computations via semidefinite programming relaxations. New Journal of
@@ -14,10 +14,11 @@ Created on Fri May 10 09:45:11 2013
 """
 import time
 from sympy.physics.quantum.dagger import Dagger
-from ncpol2sdpa import generate_variables, bosonic_constraints, SdpRelaxation
+from ncpol2sdpa import generate_variables, bosonic_constraints, SdpRelaxation,\
+                       write_to_sdpa
 
-# Order of relaxation
-order = 2
+# Level of relaxation
+level = 1
 
 # Number of variables
 N = 3
@@ -38,12 +39,12 @@ inequalities = []
 time0 = time.time()
 # Obtain SDP relaxation
 print("Obtaining SDP relaxation...")
-verbose = 1
-sdpRelaxation = SdpRelaxation(a)
+sdpRelaxation = SdpRelaxation(a, verbose=1)
 sdpRelaxation.get_relaxation(hamiltonian, inequalities, equalities,
-                             monomial_substitutions, order, verbose)
+                             monomial_substitutions, level,
+                             removeequalities=True)
 # Export relaxation to SDPA format
 print("Writing to disk...")
-sdpRelaxation.write_to_sdpa('harmonic_oscillator.dat-s')
+write_to_sdpa(sdpRelaxation, 'harmonic_oscillator.dat-s')
 
 print('%0.2f s' % ((time.time() - time0)))
