@@ -7,10 +7,12 @@ Created on Fri May 16 14:27:47 2014
 @author: Peter Wittek
 """
 from sympy.physics.quantum.dagger import Dagger
-from ncpol2sdpa import generate_variables, SdpRelaxation, solve_sdp
+from .nc_utils import generate_variables, flatten
+from .sdpa_utils import solve_sdp
+from .sdp_relaxation import SdpRelaxation
 
 
-def get_neighbors(index, lattice_length, W=0, periodic=False):
+def get_neighbors(index, lattice_length, width=0, periodic=False):
     """Get the neighbors of an operator in a lattice.
 
     Arguments:
@@ -19,22 +21,25 @@ def get_neighbors(index, lattice_length, W=0, periodic=False):
 
     Returns a list of neighbors in linear index.
     """
-    if W == 0:
-        W = lattice_length
+    if width == 0:
+        width = lattice_length
     neighbors = []
-    coords = divmod(index, W)
-    if coords[1] < W - 1:
+    coords = divmod(index, width)
+    if coords[1] < width - 1:
         neighbors.append(index + 1)
     elif periodic:
-        neighbors.append(index - W + 1)
+        neighbors.append(index - width + 1)
     if coords[0] < lattice_length - 1:
-        neighbors.append(index + W)
+        neighbors.append(index + width)
     elif periodic:
-        neighbors.append(index - (lattice_length - 1) * W)
+        neighbors.append(index - (lattice_length - 1) * width)
     return neighbors
 
 
 def bosonic_constraints(a):
+    """Return a set of constraints that define bosonic ladder
+    operators.
+    """
     n_vars = len(a)
     monomial_substitutions = {}
     equalities = []
@@ -56,10 +61,13 @@ def bosonic_constraints(a):
 
 
 def fermionic_constraints(a):
+    """Return  a set of constraints that define fermionic ladder
+    operators.
+    """
     n_vars = len(a)
     monomial_substitutions = {}
     equalities = []
-    inequalities = []    
+    inequalities = []
     for i in range(n_vars):
         for j in range(i + 1, n_vars):
             # {a_i,a_jT} = 0 for i\neq j
@@ -87,19 +95,59 @@ def fermionic_constraints(a):
         equalities.append(a[i] * Dagger(a[i]) + Dagger(a[i]) * a[i] - 1.0)
 
     for i in range(n_vars):
-        inequalities.append(Dagger(a[i])*a[i])
-        inequalities.append(1-Dagger(a[i])*a[i])
+        inequalities.append(Dagger(a[i]) * a[i])
+        inequalities.append(1 - Dagger(a[i]) * a[i])
     return monomial_substitutions, equalities, inequalities
 
 
+def pauli_constraints(X, Y, Z):
+    """Return  a set of constraints that define Pauli spin operators.
+    """
+    monomial_substitutions = {}
+    equalities = []
+    n_vars = len(X)
+    for i in range(n_vars):
+        # They square to the identity
+        monomial_substitutions[X[i] * X[i]] = 1
+        monomial_substitutions[Y[i] * Y[i]] = 1
+        monomial_substitutions[Z[i] * Z[i]] = 1
+
+        # Anticommutation relations
+        equalities.append(Y[i] * X[i] + X[i] * Y[i])
+        equalities.append(Z[i] * X[i] + X[i] * Z[i])
+        equalities.append(Z[i] * Y[i] + Y[i] * Z[i])
+        # Commutation relations.
+        #equalities.append(X[i]*Y[i] - 1j*Z[i])
+        #equalities.append(X[i]*Z[i] + 1j*Y[i])
+        #equalities.append(Y[i]*Z[i] - 1j*X[i])
+        # They commute between the sites
+        for j in range(i + 1, n_vars):
+            monomial_substitutions[X[j] * X[i]] = X[i] * X[j]
+            monomial_substitutions[Y[j] * Y[i]] = Y[i] * Y[j]
+            monomial_substitutions[Y[j] * X[i]] = X[i] * Y[j]
+            monomial_substitutions[Y[i] * X[j]] = X[j] * Y[i]
+            monomial_substitutions[Z[j] * Z[i]] = Z[i] * Z[j]
+            monomial_substitutions[Z[j] * X[i]] = X[i] * Z[j]
+            monomial_substitutions[Z[i] * X[j]] = X[j] * Z[i]
+            monomial_substitutions[Z[j] * Y[i]] = Y[i] * Z[j]
+            monomial_substitutions[Z[i] * Y[j]] = Y[j] * Z[i]
+    return monomial_substitutions, equalities
+
+
 def generate_measurements(party, label):
+    """Generate variables that behave like measurements.
+    """
     measurements = []
     for i in range(len(party)):
-        measurements.append(generate_variables(party[i]-1, hermitian=True,
+        measurements.append(generate_variables(party[i] - 1, hermitian=True,
                                                name=label + '%s' % i))
     return measurements
 
+
 def projective_measurement_constraints(A, B):
+    """Return a set of constraints that define projective
+    measurements.
+    """
     monomial_substitutions = {}
     for Mk in [M for M_list in [A, B] for M in M_list]:
         for Ei in Mk:
@@ -121,6 +169,9 @@ def projective_measurement_constraints(A, B):
 
 
 def define_objective_with_I(I, A, B):
+    """Define a polynomial using measurements and an I matrix describing a Bell
+    inequality.
+    """
     objective = 0
     i, j = 0, 1  # Row and column index in I
     for m_Bj in B:  # Define first row
@@ -134,47 +185,43 @@ def define_objective_with_I(I, A, B):
             j = 1
             for m_Bj in B:
                 for Bj in m_Bj:
-                        objective += I[i][j] * Ai * Bj
-                        j += 1
+                    objective += I[i][j] * Ai * Bj
+                    j += 1
             i += 1
     return -objective
 
 
-def flatten(lol):
-    new_list = []
-    for element in lol:
-        if isinstance(element[0], list):
-            element = flatten(element)
-        new_list.extend(element)
-    return new_list
-
 def correlator(A, B):
+    """Correlators between the probabilities of two parties.
+    """
     correlators = []
     for i in range(len(A)):
         correlator_row = []
         for j in range(len(B)):
-            correlator = 0
+            corr = 0
             for k in range(len(A[i])):
                 for l in range(len(B[j])):
                     if k == l:
-                        correlator += A[i][k] * B[j][l]
+                        corr += A[i][k] * B[j][l]
                     else:
-                        correlator -= A[i][k] * B[j][l]
-            correlator_row.append(correlator)
+                        corr -= A[i][k] * B[j][l]
+            correlator_row.append(corr)
         correlators.append(correlator_row)
     return correlators
 
 
 def maximum_violation(A_configuration, B_configuration, I, level):
+    """Get the maximum violation of a Bell inequality.
+    """
     A = generate_measurements(A_configuration, 'A')
     B = generate_measurements(B_configuration, 'B')
 
-    monomial_substitutions, equalities = projective_measurement_constraints(
+    monomial_substitutions = projective_measurement_constraints(
         A, B)
 
     objective = define_objective_with_I(I, A, B)
 
     sdpRelaxation = SdpRelaxation(flatten([A, B]), verbose=2)
-    sdpRelaxation.get_relaxation(objective, [], equalities,
+    sdpRelaxation.get_relaxation(objective, [], [],
                                  monomial_substitutions, level)
     return solve_sdp(sdpRelaxation)
